/*     */ package org.hibernate.cfg;
/*     */ 
/*     */ import java.util.Iterator;
/*     */ import java.util.Map;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.annotations.ForeignKey;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.annotations.common.reflection.XProperty;
/*     */ import org.hibernate.cfg.annotations.PropertyBinder;
/*     */ import org.hibernate.mapping.Column;
/*     */ import org.hibernate.mapping.DependantValue;
/*     */ import org.hibernate.mapping.Join;
/*     */ import org.hibernate.mapping.KeyValue;
/*     */ import org.hibernate.mapping.ManyToOne;
/*     */ import org.hibernate.mapping.OneToOne;
/*     */ import org.hibernate.mapping.PersistentClass;
/*     */ import org.hibernate.mapping.Property;
/*     */ import org.hibernate.mapping.SimpleValue;
/*     */ import org.hibernate.mapping.Value;
/*     */ import org.hibernate.type.ForeignKeyDirection;
/*     */ import org.hibernate.util.StringHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class OneToOneSecondPass
/*     */   implements SecondPass
/*     */ {
/*     */   private String mappedBy;
/*     */   private ExtendedMappings mappings;
/*     */   private String ownerEntity;
/*     */   private String ownerProperty;
/*     */   private PropertyHolder propertyHolder;
/*     */   private boolean ignoreNotFound;
/*     */   private PropertyData inferredData;
/*     */   private XClass targetEntity;
/*     */   private boolean cascadeOnDelete;
/*     */   private boolean optional;
/*     */   private String cascadeStrategy;
/*     */   private Ejb3JoinColumn[] joinColumns;
/*     */   
/*     */   public OneToOneSecondPass(String mappedBy, String ownerEntity, String ownerProperty, PropertyHolder propertyHolder, PropertyData inferredData, XClass targetEntity, boolean ignoreNotFound, boolean cascadeOnDelete, boolean optional, String cascadeStrategy, Ejb3JoinColumn[] columns, ExtendedMappings mappings)
/*     */   {
/*  48 */     this.ownerEntity = ownerEntity;
/*  49 */     this.ownerProperty = ownerProperty;
/*  50 */     this.mappedBy = mappedBy;
/*  51 */     this.propertyHolder = propertyHolder;
/*  52 */     this.mappings = mappings;
/*  53 */     this.ignoreNotFound = ignoreNotFound;
/*  54 */     this.inferredData = inferredData;
/*  55 */     this.targetEntity = targetEntity;
/*  56 */     this.cascadeOnDelete = cascadeOnDelete;
/*  57 */     this.optional = optional;
/*  58 */     this.cascadeStrategy = cascadeStrategy;
/*  59 */     this.joinColumns = columns;
/*     */   }
/*     */   
/*     */   public void doSecondPass(Map persistentClasses) throws MappingException
/*     */   {
/*  64 */     OneToOne value = new OneToOne(this.propertyHolder.getTable(), this.propertyHolder.getPersistentClass());
/*     */     
/*     */ 
/*  67 */     String propertyName = this.inferredData.getPropertyName();
/*  68 */     value.setPropertyName(propertyName);
/*     */     String referencedEntityName;
/*  70 */     String referencedEntityName; if (AnnotationBinder.isDefault(this.targetEntity, this.mappings)) {
/*  71 */       referencedEntityName = this.inferredData.getClassOrElementName();
/*     */     }
/*     */     else {
/*  74 */       referencedEntityName = this.targetEntity.getName();
/*     */     }
/*  76 */     value.setReferencedEntityName(referencedEntityName);
/*  77 */     AnnotationBinder.defineFetchingStrategy(value, this.inferredData.getProperty());
/*     */     
/*  79 */     value.setCascadeDeleteEnabled(this.cascadeOnDelete);
/*     */     
/*     */ 
/*  82 */     if (!this.optional) value.setConstrained(true);
/*  83 */     value.setForeignKeyType(value.isConstrained() ? ForeignKeyDirection.FOREIGN_KEY_FROM_PARENT : ForeignKeyDirection.FOREIGN_KEY_TO_PARENT);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  88 */     PropertyBinder binder = new PropertyBinder();
/*  89 */     binder.setName(propertyName);
/*  90 */     binder.setValue(value);
/*  91 */     binder.setCascade(this.cascadeStrategy);
/*  92 */     binder.setPropertyAccessorName(this.inferredData.getDefaultAccess());
/*  93 */     Property prop = binder.make();
/*  94 */     if (BinderHelper.isDefault(this.mappedBy))
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 101 */       boolean rightOrder = true;
/*     */       
/* 103 */       if (rightOrder) {
/* 104 */         String path = StringHelper.qualify(this.propertyHolder.getPath(), propertyName);
/* 105 */         new ToOneFkSecondPass(value, this.joinColumns, !this.optional, this.propertyHolder.getEntityOwnerClassName(), path, this.mappings).doSecondPass(persistentClasses);
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 112 */         this.propertyHolder.addProperty(prop);
/*     */       }
/*     */       
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/*     */ 
/* 120 */       PersistentClass otherSide = (PersistentClass)persistentClasses.get(value.getReferencedEntityName());
/*     */       Property otherSideProperty;
/*     */       try {
/* 123 */         if (otherSide == null) {
/* 124 */           throw new MappingException("Unable to find entity: " + value.getReferencedEntityName());
/*     */         }
/* 126 */         otherSideProperty = BinderHelper.findPropertyByName(otherSide, this.mappedBy);
/*     */       }
/*     */       catch (MappingException e) {
/* 129 */         throw new AnnotationException("Unknown mappedBy in: " + StringHelper.qualify(this.ownerEntity, this.ownerProperty) + ", referenced property unknown: " + StringHelper.qualify(value.getReferencedEntityName(), this.mappedBy));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 135 */       if (otherSideProperty == null) {
/* 136 */         throw new AnnotationException("Unknown mappedBy in: " + StringHelper.qualify(this.ownerEntity, this.ownerProperty) + ", referenced property unknown: " + StringHelper.qualify(value.getReferencedEntityName(), this.mappedBy));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 142 */       if ((otherSideProperty.getValue() instanceof OneToOne)) {
/* 143 */         this.propertyHolder.addProperty(prop);
/*     */       }
/* 145 */       else if ((otherSideProperty.getValue() instanceof ManyToOne)) {
/* 146 */         Iterator it = otherSide.getJoinIterator();
/* 147 */         Join otherSideJoin = null;
/* 148 */         while (it.hasNext()) {
/* 149 */           otherSideJoin = (Join)it.next();
/* 150 */           if (otherSideJoin.containsProperty(otherSideProperty)) {
/*     */             break;
/*     */           }
/*     */         }
/* 154 */         if (otherSideJoin != null)
/*     */         {
/* 156 */           Join mappedByJoin = buildJoin((PersistentClass)persistentClasses.get(this.ownerEntity), otherSideProperty, otherSideJoin);
/*     */           
/*     */ 
/* 159 */           ManyToOne manyToOne = new ManyToOne(mappedByJoin.getTable());
/*     */           
/* 161 */           manyToOne.setIgnoreNotFound(this.ignoreNotFound);
/* 162 */           manyToOne.setCascadeDeleteEnabled(value.isCascadeDeleteEnabled());
/* 163 */           manyToOne.setEmbedded(value.isEmbedded());
/* 164 */           manyToOne.setFetchMode(value.getFetchMode());
/* 165 */           manyToOne.setLazy(value.isLazy());
/* 166 */           manyToOne.setReferencedEntityName(value.getReferencedEntityName());
/* 167 */           manyToOne.setUnwrapProxy(value.isUnwrapProxy());
/* 168 */           prop.setValue(manyToOne);
/* 169 */           Iterator otherSideJoinKeyColumns = otherSideJoin.getKey().getColumnIterator();
/* 170 */           while (otherSideJoinKeyColumns.hasNext()) {
/* 171 */             Column column = (Column)otherSideJoinKeyColumns.next();
/* 172 */             Column copy = new Column();
/* 173 */             copy.setLength(column.getLength());
/* 174 */             copy.setScale(column.getScale());
/* 175 */             copy.setValue(manyToOne);
/* 176 */             copy.setName(column.getQuotedName());
/* 177 */             copy.setNullable(column.isNullable());
/* 178 */             copy.setPrecision(column.getPrecision());
/* 179 */             copy.setUnique(column.isUnique());
/* 180 */             copy.setSqlType(column.getSqlType());
/* 181 */             copy.setCheckConstraint(column.getCheckConstraint());
/* 182 */             copy.setComment(column.getComment());
/* 183 */             copy.setDefaultValue(column.getDefaultValue());
/* 184 */             manyToOne.addColumn(copy);
/*     */           }
/* 186 */           mappedByJoin.addProperty(prop);
/*     */         }
/*     */         else {
/* 189 */           this.propertyHolder.addProperty(prop);
/*     */         }
/*     */         
/* 192 */         value.setReferencedPropertyName(this.mappedBy);
/*     */         
/* 194 */         String propertyRef = value.getReferencedPropertyName();
/* 195 */         if (propertyRef != null) {
/* 196 */           this.mappings.addUniquePropertyReference(value.getReferencedEntityName(), propertyRef);
/*     */         }
/*     */         
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/* 203 */         throw new AnnotationException("Referenced property not a (One|Many)ToOne: " + StringHelper.qualify(otherSide.getEntityName(), this.mappedBy) + " in mappedBy of " + StringHelper.qualify(this.ownerEntity, this.ownerProperty));
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 213 */     ForeignKey fk = (ForeignKey)this.inferredData.getProperty().getAnnotation(ForeignKey.class);
/* 214 */     String fkName = fk != null ? fk.name() : "";
/* 215 */     if (!BinderHelper.isDefault(fkName)) value.setForeignKeyName(fkName);
/*     */   }
/*     */   
/*     */   private Join buildJoin(PersistentClass persistentClass, Property otherSideProperty, Join originalJoin)
/*     */   {
/* 220 */     Join join = new Join();
/* 221 */     join.setPersistentClass(persistentClass);
/*     */     
/*     */ 
/* 224 */     join.setTable(originalJoin.getTable());
/* 225 */     join.setInverse(true);
/* 226 */     SimpleValue key = new DependantValue(join.getTable(), persistentClass.getIdentifier());
/*     */     
/* 228 */     join.setKey(key);
/* 229 */     join.setSequentialSelect(false);
/*     */     
/* 231 */     join.setOptional(true);
/* 232 */     key.setCascadeDeleteEnabled(false);
/* 233 */     Iterator mappedByColumns = otherSideProperty.getValue().getColumnIterator();
/* 234 */     while (mappedByColumns.hasNext()) {
/* 235 */       Column column = (Column)mappedByColumns.next();
/* 236 */       Column copy = new Column();
/* 237 */       copy.setLength(column.getLength());
/* 238 */       copy.setScale(column.getScale());
/* 239 */       copy.setValue(key);
/* 240 */       copy.setName(column.getQuotedName());
/* 241 */       copy.setNullable(column.isNullable());
/* 242 */       copy.setPrecision(column.getPrecision());
/* 243 */       copy.setUnique(column.isUnique());
/* 244 */       copy.setSqlType(column.getSqlType());
/* 245 */       copy.setCheckConstraint(column.getCheckConstraint());
/* 246 */       copy.setComment(column.getComment());
/* 247 */       copy.setDefaultValue(column.getDefaultValue());
/* 248 */       key.addColumn(copy);
/*     */     }
/* 250 */     join.createPrimaryKey();
/* 251 */     join.createForeignKey();
/* 252 */     persistentClass.addJoin(join);
/* 253 */     return join;
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\OneToOneSecondPass.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */