/*     */ package org.hibernate.cfg.annotations;
/*     */ 
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.Random;
/*     */ import java.util.Set;
/*     */ import javax.persistence.AttributeOverride;
/*     */ import javax.persistence.AttributeOverrides;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.AssertionFailure;
/*     */ import org.hibernate.FetchMode;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.annotations.MapKey;
/*     */ import org.hibernate.annotations.MapKeyManyToMany;
/*     */ import org.hibernate.annotations.Type;
/*     */ import org.hibernate.annotations.common.reflection.ReflectionManager;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.annotations.common.reflection.XProperty;
/*     */ import org.hibernate.cfg.AnnotatedClassType;
/*     */ import org.hibernate.cfg.AnnotationBinder;
/*     */ import org.hibernate.cfg.BinderHelper;
/*     */ import org.hibernate.cfg.CollectionSecondPass;
/*     */ import org.hibernate.cfg.Ejb3Column;
/*     */ import org.hibernate.cfg.Ejb3JoinColumn;
/*     */ import org.hibernate.cfg.ExtendedMappings;
/*     */ import org.hibernate.cfg.Mappings;
/*     */ import org.hibernate.cfg.PropertyData;
/*     */ import org.hibernate.cfg.PropertyHolder;
/*     */ import org.hibernate.cfg.PropertyHolderBuilder;
/*     */ import org.hibernate.cfg.PropertyPreloadedData;
/*     */ import org.hibernate.cfg.SecondPass;
/*     */ import org.hibernate.dialect.HSQLDialect;
/*     */ import org.hibernate.mapping.Collection;
/*     */ import org.hibernate.mapping.Column;
/*     */ import org.hibernate.mapping.Component;
/*     */ import org.hibernate.mapping.DependantValue;
/*     */ import org.hibernate.mapping.Formula;
/*     */ import org.hibernate.mapping.KeyValue;
/*     */ import org.hibernate.mapping.ManyToOne;
/*     */ import org.hibernate.mapping.OneToMany;
/*     */ import org.hibernate.mapping.PersistentClass;
/*     */ import org.hibernate.mapping.Property;
/*     */ import org.hibernate.mapping.SimpleValue;
/*     */ import org.hibernate.mapping.Table;
/*     */ import org.hibernate.mapping.ToOne;
/*     */ import org.hibernate.mapping.Value;
/*     */ import org.hibernate.sql.Template;
/*     */ import org.hibernate.util.StringHelper;
/*     */ 
/*     */ public class MapBinder
/*     */   extends CollectionBinder
/*     */ {
/*     */   public MapBinder(boolean sorted)
/*     */   {
/*  55 */     super(sorted);
/*     */   }
/*     */   
/*     */ 
/*     */   public MapBinder() {}
/*     */   
/*     */   protected Collection createCollection(PersistentClass persistentClass)
/*     */   {
/*  63 */     return new org.hibernate.mapping.Map(persistentClass);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SecondPass getSecondPass(final Ejb3JoinColumn[] fkJoinColumns, final Ejb3JoinColumn[] keyColumns, final Ejb3JoinColumn[] inverseColumns, final Ejb3Column[] elementColumns, final Ejb3Column[] mapKeyColumns, final Ejb3JoinColumn[] mapKeyManyToManyColumns, final boolean isEmbedded, final XProperty property, final XClass collType, final boolean ignoreNotFound, final boolean unique, final TableBinder assocTableBinder, final ExtendedMappings mappings)
/*     */   {
/*  76 */     new CollectionSecondPass(mappings, this.collection)
/*     */     {
/*     */       public void secondPass(java.util.Map persistentClasses, java.util.Map inheritedMetas) throws MappingException {
/*  79 */         MapBinder.this.bindStarToManySecondPass(persistentClasses, collType, fkJoinColumns, keyColumns, inverseColumns, elementColumns, isEmbedded, property, unique, assocTableBinder, ignoreNotFound, mappings);
/*     */         
/*     */ 
/*     */ 
/*  83 */         MapBinder.this.bindKeyFromAssociationTable(collType, persistentClasses, MapBinder.this.mapKeyPropertyName, property, isEmbedded, mappings, mapKeyColumns, mapKeyManyToManyColumns, inverseColumns != null ? inverseColumns[0].getPropertyName() : null);
/*     */       }
/*     */     };
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void bindKeyFromAssociationTable(XClass collType, java.util.Map persistentClasses, String mapKeyPropertyName, XProperty property, boolean isEmbedded, ExtendedMappings mappings, Ejb3Column[] mapKeyColumns, Ejb3JoinColumn[] mapKeyManyToManyColumns, String targetPropertyName)
/*     */   {
/*  97 */     if (mapKeyPropertyName != null)
/*     */     {
/*  99 */       PersistentClass associatedClass = (PersistentClass)persistentClasses.get(collType.getName());
/* 100 */       if (associatedClass == null) throw new AnnotationException("Associated class not found: " + collType);
/* 101 */       Property mapProperty = BinderHelper.findPropertyByName(associatedClass, mapKeyPropertyName);
/* 102 */       if (mapProperty == null) {
/* 103 */         throw new AnnotationException("Map key property not found: " + collType + "." + mapKeyPropertyName);
/*     */       }
/*     */       
/*     */ 
/* 107 */       org.hibernate.mapping.Map map = (org.hibernate.mapping.Map)this.collection;
/* 108 */       Value indexValue = createFormulatedValue(mapProperty.getValue(), map, targetPropertyName, associatedClass);
/* 109 */       map.setIndex(indexValue);
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/*     */ 
/* 115 */       Class target = Void.TYPE;
/*     */       
/*     */ 
/*     */ 
/* 119 */       if (property.isAnnotationPresent(MapKey.class)) {
/* 120 */         target = ((MapKey)property.getAnnotation(MapKey.class)).targetElement();
/*     */       }
/* 122 */       else if (property.isAnnotationPresent(MapKeyManyToMany.class))
/* 123 */         target = ((MapKeyManyToMany)property.getAnnotation(MapKeyManyToMany.class)).targetEntity();
/*     */       String mapKeyType;
/* 125 */       String mapKeyType; if (!Void.TYPE.equals(target)) {
/* 126 */         mapKeyType = target.getName();
/*     */       }
/*     */       else {
/* 129 */         mapKeyType = property.getMapKey().getName();
/*     */       }
/* 131 */       PersistentClass collectionEntity = (PersistentClass)persistentClasses.get(mapKeyType);
/* 132 */       boolean isIndexOfEntities = collectionEntity != null;
/* 133 */       ManyToOne element = null;
/* 134 */       org.hibernate.mapping.Map mapValue = (org.hibernate.mapping.Map)this.collection;
/* 135 */       if (isIndexOfEntities) {
/* 136 */         element = new ManyToOne(mapValue.getCollectionTable());
/* 137 */         mapValue.setIndex(element);
/* 138 */         element.setReferencedEntityName(mapKeyType);
/*     */         
/*     */ 
/*     */ 
/* 142 */         element.setFetchMode(FetchMode.JOIN);
/* 143 */         element.setLazy(false);
/*     */ 
/*     */ 
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/* 153 */         PropertyHolder holder = null;
/* 154 */         XClass elementClass; XClass elementClass; AnnotatedClassType classType; if (BinderHelper.PRIMITIVE_NAMES.contains(mapKeyType)) {
/* 155 */           AnnotatedClassType classType = AnnotatedClassType.NONE;
/* 156 */           elementClass = null;
/*     */         }
/*     */         else {
/*     */           try {
/* 160 */             elementClass = mappings.getReflectionManager().classForName(mapKeyType, MapBinder.class);
/*     */           }
/*     */           catch (ClassNotFoundException e) {
/* 163 */             throw new AnnotationException("Unable to find class: " + mapKeyType, e);
/*     */           }
/* 165 */           classType = mappings.getClassType(elementClass);
/*     */           
/* 167 */           holder = PropertyHolderBuilder.buildPropertyHolder(mapValue, StringHelper.qualify(mapValue.getRole(), "mapkey"), elementClass, property, this.propertyHolder, mappings);
/*     */           
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 174 */           boolean attributeOverride = (property.isAnnotationPresent(AttributeOverride.class)) || (property.isAnnotationPresent(AttributeOverrides.class));
/*     */           
/* 176 */           if ((isEmbedded) || (attributeOverride)) {
/* 177 */             classType = AnnotatedClassType.EMBEDDABLE;
/*     */           }
/*     */         }
/*     */         
/* 181 */         if (AnnotatedClassType.EMBEDDABLE.equals(classType)) {
/* 182 */           EntityBinder entityBinder = new EntityBinder();
/* 183 */           PersistentClass owner = mapValue.getOwner();
/*     */           
/*     */           boolean isPropertyAnnotated;
/*     */           
/* 187 */           if (owner.getIdentifierProperty() != null) {
/* 188 */             isPropertyAnnotated = owner.getIdentifierProperty().getPropertyAccessorName().equals("property");
/*     */           }
/*     */           else
/*     */           {
/*     */             boolean isPropertyAnnotated;
/* 193 */             if ((owner.getIdentifierMapper() != null) && (owner.getIdentifierMapper().getPropertySpan() > 0)) {
/* 194 */               Property prop = (Property)owner.getIdentifierMapper().getPropertyIterator().next();
/* 195 */               isPropertyAnnotated = prop.getPropertyAccessorName().equals("property");
/*     */             }
/*     */             else {
/* 198 */               throw new AssertionFailure("Unable to guess collection property accessor name");
/*     */             }
/*     */           }
/*     */           
/*     */           boolean isPropertyAnnotated;
/* 203 */           PropertyData inferredData = new PropertyPreloadedData("property", "index", elementClass);
/*     */           
/* 205 */           Component component = AnnotationBinder.fillComponent(holder, inferredData, isPropertyAnnotated, isPropertyAnnotated ? "property" : "field", true, entityBinder, false, false, true, mappings);
/*     */           
/*     */ 
/*     */ 
/*     */ 
/* 210 */           mapValue.setIndex(component);
/*     */         }
/*     */         else {
/* 213 */           SimpleValueBinder elementBinder = new SimpleValueBinder();
/* 214 */           elementBinder.setMappings(mappings);
/* 215 */           elementBinder.setReturnedClassName(mapKeyType);
/*     */           
/* 217 */           Ejb3Column[] elementColumns = mapKeyColumns;
/* 218 */           if ((elementColumns == null) || (elementColumns.length == 0)) {
/* 219 */             elementColumns = new Ejb3Column[1];
/* 220 */             Ejb3Column column = new Ejb3Column();
/* 221 */             column.setImplicit(false);
/* 222 */             column.setNullable(true);
/* 223 */             column.setLength(255);
/* 224 */             column.setLogicalColumnName("id");
/*     */             
/* 226 */             column.setJoins(new HashMap());
/* 227 */             column.setMappings(mappings);
/* 228 */             column.bind();
/* 229 */             elementColumns[0] = column;
/*     */           }
/*     */           
/* 232 */           for (Ejb3Column column : elementColumns) {
/* 233 */             column.setTable(mapValue.getCollectionTable());
/*     */           }
/* 235 */           elementBinder.setColumns(elementColumns);
/*     */           
/*     */ 
/* 238 */           MapKey mapKeyAnn = (MapKey)property.getAnnotation(MapKey.class);
/* 239 */           if ((mapKeyAnn != null) && (!BinderHelper.isDefault(mapKeyAnn.type().type()))) {
/* 240 */             elementBinder.setExplicitType(mapKeyAnn.type());
/*     */           }
/* 242 */           mapValue.setIndex(elementBinder.make());
/*     */         }
/*     */       }
/*     */       
/* 246 */       if (!this.collection.isOneToMany())
/*     */       {
/* 248 */         for (Ejb3JoinColumn col : mapKeyManyToManyColumns) {
/* 249 */           col.forceNotNull();
/*     */         }
/*     */       }
/* 252 */       if (isIndexOfEntities) {
/* 253 */         bindManytoManyInverseFk(collectionEntity, mapKeyManyToManyColumns, element, false, mappings);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Value createFormulatedValue(Value value, Collection collection, String targetPropertyName, PersistentClass associatedClass)
/*     */   {
/* 267 */     Value element = collection.getElement();
/* 268 */     String fromAndWhere = null;
/* 269 */     if (!(element instanceof OneToMany)) {
/* 270 */       String referencedPropertyName = null;
/* 271 */       if ((element instanceof ToOne)) {
/* 272 */         referencedPropertyName = ((ToOne)element).getReferencedPropertyName();
/*     */       }
/* 274 */       else if ((element instanceof DependantValue))
/*     */       {
/* 276 */         if (this.propertyName != null) {
/* 277 */           referencedPropertyName = collection.getReferencedPropertyName();
/*     */         }
/*     */         else
/* 280 */           throw new AnnotationException("SecondaryTable JoinColumn cannot reference a non primary key");
/*     */       }
/*     */       Iterator referencedEntityColumns;
/*     */       Iterator referencedEntityColumns;
/* 284 */       if (referencedPropertyName == null) {
/* 285 */         referencedEntityColumns = associatedClass.getIdentifier().getColumnIterator();
/*     */       }
/*     */       else {
/* 288 */         Property referencedProperty = associatedClass.getRecursiveProperty(referencedPropertyName);
/* 289 */         referencedEntityColumns = referencedProperty.getColumnIterator();
/*     */       }
/* 291 */       String alias = "$alias$";
/* 292 */       StringBuilder fromAndWhereSb = new StringBuilder(" from ").append(associatedClass.getTable().getName()).append(" ").append(alias).append(" where ");
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 297 */       Iterator collectionTableColumns = element.getColumnIterator();
/* 298 */       while (collectionTableColumns.hasNext()) {
/* 299 */         Column colColumn = (Column)collectionTableColumns.next();
/* 300 */         Column refColumn = (Column)referencedEntityColumns.next();
/* 301 */         fromAndWhereSb.append(alias).append('.').append(refColumn.getQuotedName()).append('=').append(colColumn.getQuotedName()).append(" and ");
/*     */       }
/*     */       
/* 304 */       fromAndWhere = fromAndWhereSb.substring(0, fromAndWhereSb.length() - 5);
/*     */     }
/*     */     
/* 307 */     if ((value instanceof Component)) {
/* 308 */       Component component = (Component)value;
/* 309 */       Iterator properties = component.getPropertyIterator();
/* 310 */       Component indexComponent = new Component(collection);
/* 311 */       indexComponent.setComponentClassName(component.getComponentClassName());
/*     */       
/* 313 */       indexComponent.setNodeName("index");
/* 314 */       while (properties.hasNext()) {
/* 315 */         Property current = (Property)properties.next();
/* 316 */         Property newProperty = new Property();
/* 317 */         newProperty.setCascade(current.getCascade());
/* 318 */         newProperty.setGeneration(current.getGeneration());
/* 319 */         newProperty.setInsertable(false);
/* 320 */         newProperty.setUpdateable(false);
/* 321 */         newProperty.setMetaAttributes(current.getMetaAttributes());
/* 322 */         newProperty.setName(current.getName());
/* 323 */         newProperty.setNodeName(current.getNodeName());
/* 324 */         newProperty.setNaturalIdentifier(false);
/*     */         
/* 326 */         newProperty.setOptional(false);
/* 327 */         newProperty.setPersistentClass(current.getPersistentClass());
/* 328 */         newProperty.setPropertyAccessorName(current.getPropertyAccessorName());
/* 329 */         newProperty.setSelectable(current.isSelectable());
/* 330 */         newProperty.setValue(createFormulatedValue(current.getValue(), collection, targetPropertyName, associatedClass));
/*     */         
/*     */ 
/* 333 */         indexComponent.addProperty(newProperty);
/*     */       }
/* 335 */       return indexComponent;
/*     */     }
/* 337 */     if ((value instanceof SimpleValue)) {
/* 338 */       SimpleValue sourceValue = (SimpleValue)value;
/*     */       SimpleValue targetValue;
/* 340 */       SimpleValue targetValue; if ((value instanceof ManyToOne)) {
/* 341 */         ManyToOne sourceManyToOne = (ManyToOne)sourceValue;
/* 342 */         ManyToOne targetManyToOne = new ManyToOne(collection.getCollectionTable());
/* 343 */         targetManyToOne.setFetchMode(FetchMode.DEFAULT);
/* 344 */         targetManyToOne.setLazy(true);
/*     */         
/* 346 */         targetManyToOne.setReferencedEntityName(sourceManyToOne.getReferencedEntityName());
/* 347 */         targetValue = targetManyToOne;
/*     */       }
/*     */       else {
/* 350 */         targetValue = new SimpleValue(collection.getCollectionTable());
/* 351 */         targetValue.setTypeName(sourceValue.getTypeName());
/* 352 */         targetValue.setTypeParameters(sourceValue.getTypeParameters());
/*     */       }
/* 354 */       Iterator columns = sourceValue.getColumnIterator();
/* 355 */       Random random = new Random();
/* 356 */       while (columns.hasNext()) {
/* 357 */         Object current = columns.next();
/* 358 */         Formula formula = new Formula();
/*     */         String formulaString;
/* 360 */         if ((current instanceof Column)) {
/* 361 */           formulaString = ((Column)current).getQuotedName();
/*     */         } else { String formulaString;
/* 363 */           if ((current instanceof Formula)) {
/* 364 */             formulaString = ((Formula)current).getFormula();
/*     */           }
/*     */           else
/* 367 */             throw new AssertionFailure("Unknown element in column iterator: " + current.getClass()); }
/*     */         String formulaString;
/* 369 */         if (fromAndWhere != null) {
/* 370 */           formulaString = Template.renderWhereStringTemplate(formulaString, "$alias$", new HSQLDialect());
/* 371 */           formulaString = "(select " + formulaString + fromAndWhere + ")";
/* 372 */           formulaString = StringHelper.replace(formulaString, "$alias$", "a" + random.nextInt(16));
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 378 */         formula.setFormula(formulaString);
/* 379 */         targetValue.addFormula(formula);
/*     */       }
/*     */       
/* 382 */       return targetValue;
/*     */     }
/*     */     
/* 385 */     throw new AssertionFailure("Unknown type encounters for map key: " + value.getClass());
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\MapBinder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */