package com.netapp.myasup.reports.design;

import java.awt.Color;

import net.sf.jasperreports.engine.design.JRDesignExpression;
import net.sf.jasperreports.engine.design.JRDesignField;
import net.sf.jasperreports.engine.design.JRDesignFrame;
import net.sf.jasperreports.engine.design.JRDesignStyle;
import net.sf.jasperreports.engine.design.JRDesignTextField;
import net.sf.jasperreports.engine.type.EvaluationTimeEnum;
import net.sf.jasperreports.engine.type.HorizontalAlignEnum;
import net.sf.jasperreports.engine.type.HyperlinkTargetEnum;
import net.sf.jasperreports.engine.type.HyperlinkTypeEnum;
import net.sf.jasperreports.engine.type.JREnum;
import net.sf.jasperreports.engine.type.LineStyleEnum;
import net.sf.jasperreports.engine.type.ModeEnum;
import net.sf.jasperreports.engine.type.PositionTypeEnum;
import net.sf.jasperreports.engine.type.StretchTypeEnum;

import org.apache.log4j.Logger;

/**
 * This class contains common methods used in report design
 */
public class GenericReportCreator {

	private Logger log = Logger.getLogger(this.getClass());

    /**
     * Creates a style
     * @param styleName The style name
     * @param fontName Font name
     * @param foreColor Forecolor
     * @param backColor Backcolor
     * @param fontSize Font size
     * @param bold IsBold
     * @param italic IsItalic
     * @param underline IsUnderline
     * @return The style
     */
    protected JRDesignStyle createStyle(
            String styleName,
            String fontName,
            Color foreColor,
            Color backColor,
            int fontSize,
            boolean bold,
            boolean italic,
            boolean underline){
        JRDesignStyle style =new JRDesignStyle();
        //title style
        style.setName(styleName);
        style.setDefault(true);
        style.setFontName(fontName);
        style.setForecolor(foreColor);
        style.setBackcolor(backColor);
        style.setFontSize(fontSize);
        style.setBold(bold);
        style.setItalic(italic);
        style.setUnderline(underline);
        style.setStrikeThrough(false);
        style.setPdfEmbedded(true);
        return(style);
    }
    /**
     * Create a textfield
     * @param x X position
     * @param y Y position
     * @param width Width
     * @param height Height
     * @param fieldId Id for the textfield
     * @param fieldClass Field type class
     * @param expressionString Expression for the textfield
     * @param alignment The text alignment
     * @param detailStyle Style
     * @param hasBorder If the textField has border
     * @return The textfield
     */
    protected JRDesignTextField createTextField(int x, int y, int width, int height, String fieldId,
                                              String fieldClass, String expressionString, JREnum alignment,
                                              JRDesignStyle detailStyle, boolean hasBorder){
        //Create a TextField (textFieldTag attributes)
        JRDesignTextField textField=new JRDesignTextField();
        textField.setStretchWithOverflow(true);
        textField.setBlankWhenNull(true);
        textField.setEvaluationTime(EvaluationTimeEnum.NOW);
        textField.setHyperlinkType(HyperlinkTypeEnum.NONE);
        textField.setHyperlinkTarget(HyperlinkTargetEnum.SELF);

        textField.setStyle(detailStyle);
        textField.setX(x);
        textField.setY(y);
        textField.setWidth(width);
        textField.setHeight(height);
        textField.setKey(fieldId);
        textField.setHorizontalAlignment((HorizontalAlignEnum)alignment);
        textField.setStretchType(StretchTypeEnum.RELATIVE_TO_TALLEST_OBJECT);
        textField.setPositionType(PositionTypeEnum.FLOAT);
        textField.setMode(ModeEnum.OPAQUE);
        //box tag properties
        textField.getLineBox().setPadding(2);
        if(hasBorder){
            textField.getLineBox().getPen().setLineWidth(1);
            textField.getLineBox().getPen().setLineColor(Color.BLACK);
            textField.getLineBox().getPen().setLineStyle(LineStyleEnum.SOLID);
        }
        //TextField expression
        JRDesignExpression designExpression=new JRDesignExpression();
        designExpression.setValueClassName(fieldClass);
        designExpression.setText(expressionString);

        //Setting the expression
        textField.setExpression(designExpression);
        return(textField);
    }
    /**
     * Create a textfield
     * @param x X position
     * @param y Y position
     * @param width Width
     * @param height Height
     * @param fieldId Id for the textfield
     * @param fieldClass Field type class
     * @param expressionString Expression for the textfield
     * @param alignment The text alignment
     * @param detailStyle Style
     * @return The textfield
     */
    protected JRDesignTextField createTextField(int x, int y, int width, int height, String fieldId,
                                              String fieldClass, String expressionString, JREnum alignment,
                                              JRDesignStyle detailStyle){
        return createTextField(x,y,width,height,fieldId,fieldClass,expressionString,
                               alignment,detailStyle,true);
    }

    /**
     * Create a report field for the query
     * @param fieldName The field name (alias)
     * @param fieldClass The field type class name
     * @param columnName The column name in tqhe query (its number, is the column order in the query result)
     * @return the report field
     */
    protected JRDesignField createReportField(String fieldName, String fieldClass, String columnName){
        JRDesignField field=new JRDesignField();
        field.setName(fieldName);
        field.setValueClassName(fieldClass);
        field.setDescription(columnName);
        return(field);
    }
    /**
     * Creates a frame
     * @param x The x position
     * @param y The y position
     * @param width Width
     * @param height Height
     * @param frameId The identifier
     * @return The empty frame
     */
    protected JRDesignFrame createFrame(int x, int y, int width, int height, String frameId){
        //Create a frame
        JRDesignFrame frame=new JRDesignFrame();

        frame.setX(x);
        frame.setY(y);
        frame.setWidth(width);
        frame.setHeight(height);
        frame.setKey(frameId);

        return(frame);
    }
}
